#pragma once

#include "sqlite3.h"
#include "common/software_version.h"

#include <string>
#include <memory>
#include <list>
#include <vector>

// forward declaration
////////////////////////////////////////
class FileDeviceZip;
class ItemBoard;


class ModuleInfo
{
public:
    ModuleInfo( const uint32_t id, const uint8_t revision, const uint8_t assemblyIndex );

    uint32_t moduleId();
    uint8_t revision();
    uint8_t assemblyIndex();

private:
    uint32_t m_moduleId{};
    uint8_t m_revision{};
    uint8_t m_assemblyIndex{};
};


class DeviceUpdate
{
public:
    enum class DBPathType
    {
        Undefined = 0,
        Local,
        Uri
    };

    enum class DBUnzipResult
    {
        Ok,
        Failed,
        Ignore  // Archiv ohne Datenbank
    };

public:
    DeviceUpdate();
    DeviceUpdate( const std::string &pathToDB, DBPathType type = DBPathType::Local );
    virtual ~DeviceUpdate();

    /**
    * Aktualisierung der Pfadangabe fr den Datenbankzugriff.
    * Wenn eine Verbindung zur Datenbank hergestellt wird und die Pfadangabe unterschiedlich ist, dann
    * wird die aktive Verbindung geschlossen und die Verbindung zum neuen Ziel hergestellt.
    *
    * @warning Der Vergleich bercksichtigt die Gro- und Kleinschreibung der Pfadangabe.
    *
    * @param pathToDB Beinhaltet die Pfadangabe zur Datenbankdatei.
    * @param type Beschreibt den Typ der Pfaddefinition.
    * @param errorMsg Enthlt eine Beschreibung fr den Fall eines Fehlers.
    *
    * @return true - Aktualisierung / eventuelle Neuverbindung erfolgreich, false - Fehler / Datenbankverbindung getrennt
    */
    bool updatePathToDB( const std::string &pathToDB, DBPathType type, std::string &errorMsg );
    void resetInternalData();
    
    //DeviceUpdate::SyncResult syncDB( FileDeviceZip &zipFile, const std::string filePath, std::string &errorMsg );
    DBUnzipResult unzipDB( FileDeviceZip &zipFile, const std::string filePath, std::string &errorMsg );
    bool open( std::string &errorMsg );
    void close( std::string &errorMsg );
    bool isAvailable();
    bool prepareStatements();
    //std::string localDBPath();
    const std::string &dbPath() const;

    bool addFwVersion( const SoftwarebuildVersion &version, const std::string &fileName );
    const std::string &latestFwFile () const;
    bool detectMinFWVersion( ItemBoard &item, std::list<std::pair<std::string, std::string>> &moduleList );

    void updateMinFwVersion( const std::string &version );
    const SoftwarebuildVersion &minFwVersion() const;

    bool collectBlacklistDefinition( ItemBoard &item );
    bool isVersionBlacklisted( const std::string &version );
    const std::list<std::string> &blacklistVersions() const;

private:
    static std::vector<char> createAssemblyIndexArray();
    std::string selectValue( sqlite3 *db, const char *sql );

private:
    static std::vector<char> s_assemblyIndexArray;

private:
    std::string m_pathToDB;
    DBPathType m_pathType{ DBPathType::Local };
    sqlite3 *m_db{};
    sqlite3_stmt *m_stmtGetModuleMinVersion{};
    sqlite3_stmt *m_stmtGetModuleBlacklistEntries{};

    SoftwarebuildVersion m_minVersion;                      // minimale Version der aktuellen Prfung
    std::map<SoftwarebuildVersion, std::string> m_fwList;   // Liste der erlaubten Firmwareversionen (inkl. Dateiname im dfz-Container)
    std::list<std::string> m_dbBlacklist;                   // Liste der nicht erlaubten Firmwareversionen (definiert in der Datenbank)
};
